package com.macro.mall.tiny.common.util;

import cn.hutool.json.JSONObject;
import com.alibaba.fastjson.JSON;
import com.keystar.eprdms.core.result.ResponseResult;
import com.keystar.eprdms.core.shiro.entity.UserPermissionEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Map;

/**
 * @author gjs
 * @description
 * @date 2022/1/9 15:32
 */
@Component
@Slf4j
public class EprdmsHttpUtils {

    private static RestTemplate restTemplate;

    @Autowired
    public void setRedis(RestTemplate restTemplate){
        EprdmsHttpUtils.restTemplate = restTemplate;
    }

    public static <T> T getForEprdms(String requestUrl, Map<String, String> paramMap, Class<T> responseType) {
        ResponseEntity<JSONObject> responseEntity = restTemplate.getForEntity(requestUrl, JSONObject.class, paramMap);
        JSONObject entityBody = responseEntity.getBody();
        if (entityBody != null && ResponseResult.SUCCESSFUL_CODE.equals(entityBody.get("code"))) {
            return entityBody.get("data", responseType);
        } else {
            log.error("远程请求失败，requestUrl：{}，paramMap：{}", requestUrl, paramMap.toString());
            return null;
        }
    }

    public static <T> T PostForEprdms(String requestUrl, JSONObject params, Class<T> responseType) {
        HttpHeaders headers = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        headers.setContentType(type);
        headers.add("Accept", MediaType.APPLICATION_JSON.toString());

        HttpEntity<String> formEntity = new HttpEntity<>(params.toString(), headers);
        ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(requestUrl, formEntity, JSONObject.class);
        JSONObject entityBody = responseEntity.getBody();
        if (entityBody != null && ResponseResult.SUCCESSFUL_CODE.equals(entityBody.get("code"))) {
            return entityBody.get("data", responseType);
        } else {
            log.error("远程请求失败，requestUrl：{}，params：{}", requestUrl, params.toString());
            return null;
        }
    }
}
