package com.macro.mall.tiny.modules.business.controller;


import com.macro.mall.tiny.common.api.CommonResult;
import com.macro.mall.tiny.modules.business.model.BFile;
import com.macro.mall.tiny.modules.business.model.KrBLine;
import com.macro.mall.tiny.modules.business.service.KrBLineService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.websocket.server.PathParam;
import java.util.List;

/**
 * <p>
 * 线路表 前端控制器
 * </p>
 *
 * @author macro
 * @since 2021-03-24
 */
@RestController
@Api(tags = "KrBLineController", description = "提供线路相关API")
@RequestMapping("/line")
public class KrBLineController {

    @Autowired
    private KrBLineService krBLineService;


    @ApiOperation("按照ID获取线路")
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<KrBLine> getItem(@PathVariable Long id) {
        KrBLine line = krBLineService.getById(id);
        return CommonResult.success(line);
    }

    @ApiOperation("按照ID获取线路(含杆塔)")
    @RequestMapping(value = "/detail/{id}", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<KrBLine> getItemDetail(@PathVariable Long id) {
        KrBLine line = krBLineService.findById(id);
        return CommonResult.success(line);
    }

    @ApiOperation("获取线路列表")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<List<KrBLine>> list() {
        List<KrBLine> lineList = krBLineService.getLineList();
        return CommonResult.success(lineList);
    }

    @ApiOperation("按照关键字查询线路列表")
    @RequestMapping(value = "/findByKey", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<List<KrBLine>> findByKey(@RequestParam @PathParam(value = "key") String key,
                                                 @RequestParam(required = false) @PathParam(value = "provinceId") String provinceId,
                                                 @RequestParam(required = false) @PathParam(value = "cityId") String cityId) {
        List<KrBLine> lineList = krBLineService.findByKey(key, provinceId, cityId);
        return CommonResult.success(lineList);
    }

    @ApiOperation("按照关键字查询线路列表(不含杆塔)")
    @RequestMapping(value = "/findByKeySimple", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<List<KrBLine>> findByKeySimple(@RequestParam @PathParam(value = "key") String key,
                                                 @RequestParam(required = false) @PathParam(value = "provinceId") String provinceId,
                                                 @RequestParam(required = false) @PathParam(value = "cityId") String cityId) {
        List<KrBLine> lineList = krBLineService.findByKeySimple(key, provinceId, cityId);
        return CommonResult.success(lineList);
    }

    @ApiOperation("获取线路文件资料")
    @RequestMapping(value = "/getFile/{id}", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<List<BFile>> getFile(@PathVariable Long id) {
        List<BFile> data = krBLineService.getFile(id);
        return CommonResult.success(data);
    }
}

