package com.macro.mall.tiny.modules.business.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.macro.mall.tiny.common.exception.ApiException;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.CommonUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.enums.DirectoryEnum;
import com.macro.mall.tiny.modules.business.model.*;
import com.macro.mall.tiny.modules.business.service.*;
import com.macro.mall.tiny.security.util.ShiroAuthUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author gjs
 * @description
 * @date 2021/3/28 17:44
 */
@Component
@Slf4j
public class AsyncHandler {

    @Autowired
    private KrBRegionProvinceServiceImpl krBRegionProvinceService;

    @Autowired
    private KrBLineService krBLineService;

    @Autowired
    private KrBLineUploadLogService lineUploadLogService;

    @Autowired
    private KrBTowerService krBTowerService;

    @Autowired
    private KrBLineCacheService krBLineCacheService;

    @Autowired
    private BDirectoryCacheService directoryCacheService;

    @Async
    @Transactional
    public void handleDocument(String provinceId, String cityId, String fileSource, String lineName, String voltage, String lineStatus, Long serviceBegintimeTs, Integer beginTower, Integer endTower, String username, Subject subject, String charset) {
        StringBuilder warnMsg = new StringBuilder();
        KrBLine line = null;
        try {
            if (provinceId == null) Asserts.fail("上传失败，省份不能为空，provinceId：" + provinceId);
            if (cityId == null) Asserts.fail("上传失败，城市不能为空，cityId：" + cityId);
            if (username == null) Asserts.fail("上传失败，未获取到当前用户");

            LambdaQueryWrapper<KrBRegionProvince> provinceWrapper = new QueryWrapper<KrBRegionProvince>().lambda().eq(KrBRegionProvince::getProvinceId, provinceId);
            KrBRegionProvince province = krBRegionProvinceService.getOne(provinceWrapper);
            String provinceName = province.getName();

//            String username = userDetails.getUsername();
//            UmsAdmin umsAdmin = adminService.getAdminByUsername(username);
//            userId = umsAdmin.getId();
//            boolean canCover = AuthUtil.checkAccess(userDetails, AuthUtil.COVER);
//            username = UserContextHolder.getInstance().getUsername();
            boolean canCover = ShiroAuthUtil.checkAccess(subject, ShiroAuthUtil.EDIT);
            // 查看是否为线路创建者
            QueryWrapper<KrBLine> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(KrBLine::getLineName, lineName);
            line = krBLineService.getOne(wrapper);
            List<KrBTower> towerList = Lists.newArrayList();
            if (line == null) {
                // 如果为空，则创建新线路
                KrBLine newLine = new KrBLine();
                newLine.setProvinceId(provinceId);
                newLine.setCityId(cityId);
                newLine.setLineName(lineName);
                newLine.setVoltage(voltage);
                newLine.setLineStatus(lineStatus);
                newLine.setServiceBegintime(new Date(serviceBegintimeTs));
                newLine.setBeginTower(beginTower);
                newLine.setEndTower(endTower);
                newLine.setCreatorName(username);
                newLine.setCreateTime(new Date());
                krBLineService.save(newLine);
                line = newLine;

                canCover = true;
            } else if (line.getCreatorName().equals(username)) {
                canCover = true;
            }

            // 先获取根目录
            Path rootPath = Paths.get(UploadConfig.powerPath + provinceName);
            if (!Files.exists(rootPath)) {
                Files.createDirectories(rootPath);
            }

            // 获取枚举类判断
            DirectoryEnum[] enums = DirectoryEnum.values();
            HashMap<String, Boolean> booleanHashMap = new HashMap<>();
            for (DirectoryEnum directory : enums) {
                booleanHashMap.put(directory.getName(), false);
            }

            try (InputStream fis = Files.newInputStream(Paths.get(fileSource));
                 InputStream bis = new BufferedInputStream(fis);
                 ArchiveInputStream ais = new ZipArchiveInputStream(bis, charset)) {

                ArchiveEntry entry;
                while (Objects.nonNull(entry = ais.getNextEntry())) {
                    if (!ais.canReadEntryData(entry)) {
                        continue;
                    }

                    String entryName = entry.getName();
                    // 解析各项路径
                    String[] pathStr = entryName.split(UploadConfig.SEPARATOR);
                    if (pathStr.length < 2) {
                        continue;
                    }
                    entryName = lineName + entryName.substring(entryName.indexOf(UploadConfig.SEPARATOR));
                    String SecDirectoryName = pathStr[1].charAt(0) == UploadConfig.AUTH_STR ? pathStr[1].substring(1) : pathStr[1];
                    if (booleanHashMap.get(SecDirectoryName) != null) {
                        booleanHashMap.put(SecDirectoryName, true);
                    } else if ("杆塔信息录入.xlsx".equals(SecDirectoryName) || "杆塔信息录入.xls".equals(SecDirectoryName)) {
                        List<TowerExcelModel> modelArrayList = EasyExcel.read(ais, TowerExcelModel.class, null).sheet().doReadSync();
                        KrBLine finalLine = line;
                        String finalUserName = username;
                        towerList = modelArrayList.stream().map(model -> {
                            KrBTower tower = new KrBTower();
                            tower.setLineId(finalLine.getId());
                            tower.setTowerIndex(model.getTowerIndex());
                            tower.setTowerName(model.getTowerName());
                            tower.setTowerNumber(model.getTowerNumber());
                            tower.setHardwareType(model.getHardwareType());
                            // 经纬度设置
                            String lonLat = model.getLonLat();
                            String[] lonLats = lonLat.split(",");
                            BigDecimal lon = CommonUtils.Dms2D(lonLats[0]);
                            BigDecimal lat = CommonUtils.Dms2D(lonLats[1]);
                            tower.setLongitude(lon);
                            tower.setLatitude(lat);
                            tower.setType(model.getType());
                            tower.setIsStation("是".equals(model.getIsStation()) ? 1 : 0);
                            tower.setIsWeather("是".equals(model.getIsWeather()) ? 1 : 0);
                            tower.setCircuit(model.getCircuit());
                            tower.setHeight(model.getHeight());
                            tower.setPracticalHeight(model.getPracticalHeight());
                            tower.setCornerDirection(model.getCornerDirection());
                            tower.setCornerNumber(model.getCornerNumber());
                            tower.setSpan(model.getSpan());
                            tower.setServiceBegintime(model.getServiceBegintime());
                            tower.setRemark(model.getRemark());
                            tower.setHasFile(0);
                            tower.setCreatorName(finalUserName);
                            tower.setCreateTime(new Date());
                            return tower;
                        }).collect(Collectors.toList());
//                            towerService.saveAndUpdate(towerList);
                        continue;
                    } else {
                        warnMsg.append(entryName).append("第二层目录解析出错;");
                        continue;
                    }

                    Path entryFile = rootPath.resolve(entryName);
                    if (entry.isDirectory()) {
                        if (!Files.isDirectory(entryFile)) {
                            Files.createDirectories(entryFile);
                        }
                    } else {
                        Path parent = entryFile.getParent();
                        if (!canCover && Files.exists(entryFile)) {
                            warnMsg.append(entryName).append(entryFile.getFileName().toString()).append("已存在，无权覆盖;");
                        } else {
                            Files.createDirectories(parent);
                            try (OutputStream o = Files.newOutputStream(entryFile)) {
                                IOUtils.copy(ais, o);
                            }
                        }
                    }
                }
            }
            booleanHashMap.forEach((key, val) -> {
                if (!val) {
                    warnMsg.append("缺少").append(key).append("文件夹;");
                }
            });
            // 存储上传记录
            KrBLineUploadLog lineUploadLog = new KrBLineUploadLog();
            lineUploadLog.setLineId(line.getId());
            lineUploadLog.setStatus(1);
            lineUploadLog.setUploaderName(username);
            lineUploadLog.setUploadTime(new Date());
            lineUploadLog.setWarnMsg(warnMsg.toString());
            lineUploadLogService.save(lineUploadLog);

            // 后续更新塔是否包含资料
            QueryWrapper<KrBTower> towerWrapper = new QueryWrapper<>();
            towerWrapper.lambda().eq(KrBTower::getLineId, line.getId());
            List<KrBTower> oriTowerList = krBTowerService.list(towerWrapper);
            HashSet<KrBTower> towerSet = new HashSet<>(oriTowerList);
            towerSet.addAll(towerList);
            krBTowerService.setHasFile(new ArrayList<>(towerSet), subject);

            // 后续清楚缓存处理
            krBLineCacheService.delLineList();
            directoryCacheService.delDirectory();
        } catch (ApiException e) {
            log.error("上传文件抛出业务异常", e);
        } catch (Exception e) {
            log.error("上传文件抛出异常", e);
            if (line != null && username != null) {
                // 存储上传记录
                KrBLineUploadLog lineUploadLog = new KrBLineUploadLog();
                lineUploadLog.setLineId(line.getId());
                lineUploadLog.setStatus(0);
                lineUploadLog.setUploaderName(username);
                lineUploadLog.setUploadTime(new Date());
                lineUploadLog.setWarnMsg(e.getMessage());
                lineUploadLogService.save(lineUploadLog);
            }
        } finally {
            File file = new File(fileSource);
            if (file.exists()) {
                file.delete();
            }
        }
    }
}
